<?php

namespace Moblyze\Assets\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Assets\Http\Requests\AssetsRequest;
use Moblyze\Assets\Interfaces\AssetsRepositoryInterface;
use Moblyze\Assets\Models\Assets;

/**
 * Resource controller class for assets.
 */
class AssetsResourceController extends BaseController
{

    /**
     * Initialize assets resource controller.
     *
     * @param type AssetsRepositoryInterface $assets
     *
     * @return null
     */
    public function __construct(AssetsRepositoryInterface $assets)
    {
        parent::__construct();
        $this->repository = $assets;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Assets\Repositories\Criteria\AssetsResourceCriteria::class);
    }

    /**
     * Display a list of assets.
     *
     * @return Response
     */
    public function index(AssetsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Assets\Repositories\Presenter\AssetsPresenter::class)
                ->$function();
        }

        $assets = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('assets::assets.names'))
            ->view('assets::assets.index', true)
            ->data(compact('assets', 'view'))
            ->output();
    }

    /**
     * Display assets.
     *
     * @param Request $request
     * @param Model   $assets
     *
     * @return Response
     */
    public function show(AssetsRequest $request, Assets $assets)
    {

        if ($assets->exists) {
            $view = 'assets::assets.show';
        } else {
            $view = 'assets::assets.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('assets::assets.name'))
            ->data(compact('assets'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new assets.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AssetsRequest $request)
    {

        $assets = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('assets::assets.name')) 
            ->view('assets::assets.create', true) 
            ->data(compact('assets'))
            ->output();
    }

    /**
     * Create new assets.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AssetsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $assets                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('assets::assets.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('assets/assets/' . $assets->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/assets/assets'))
                ->redirect();
        }

    }

    /**
     * Show assets for editing.
     *
     * @param Request $request
     * @param Model   $assets
     *
     * @return Response
     */
    public function edit(AssetsRequest $request, Assets $assets)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('assets::assets.name'))
            ->view('assets::assets.edit', true)
            ->data(compact('assets'))
            ->output();
    }

    /**
     * Update the assets.
     *
     * @param Request $request
     * @param Model   $assets
     *
     * @return Response
     */
    public function update(AssetsRequest $request, Assets $assets)
    {
        try {
            $attributes = $request->all();

            $assets->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('assets::assets.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('assets/assets/' . $assets->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('assets/assets/' . $assets->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the assets.
     *
     * @param Model   $assets
     *
     * @return Response
     */
    public function destroy(AssetsRequest $request, Assets $assets)
    {
        try {

            $assets->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('assets::assets.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('assets/assets/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('assets/assets/' . $assets->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple assets.
     *
     * @param Model   $assets
     *
     * @return Response
     */
    public function delete(AssetsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('assets::assets.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('assets/assets'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/assets/assets'))
                ->redirect();
        }

    }

    /**
     * Restore deleted assets.
     *
     * @param Model   $assets
     *
     * @return Response
     */
    public function restore(AssetsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('assets::assets.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/assets/assets'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/assets/assets/'))
                ->redirect();
        }

    }

}
