<?php

namespace Moblyze\Assets\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Assets\Http\Requests\TypeRequest;
use Moblyze\Assets\Interfaces\TypeRepositoryInterface;
use Moblyze\Assets\Models\Type;

/**
 * Resource controller class for type.
 */
class TypeResourceController extends BaseController
{

    /**
     * Initialize type resource controller.
     *
     * @param type TypeRepositoryInterface $type
     *
     * @return null
     */
    public function __construct(TypeRepositoryInterface $type)
    {
        parent::__construct();
        $this->repository = $type;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Assets\Repositories\Criteria\TypeResourceCriteria::class);
    }

    /**
     * Display a list of type.
     *
     * @return Response
     */
    public function index(TypeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Assets\Repositories\Presenter\TypePresenter::class)
                ->$function();
        }

        $types = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('assets::type.names'))
            ->view('assets::type.index', true)
            ->data(compact('types', 'view'))
            ->output();
    }

    /**
     * Display type.
     *
     * @param Request $request
     * @param Model   $type
     *
     * @return Response
     */
    public function show(TypeRequest $request, Type $type)
    {

        if ($type->exists) {
            $view = 'assets::type.show';
        } else {
            $view = 'assets::type.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('assets::type.name'))
            ->data(compact('type'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new type.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TypeRequest $request)
    {

        $type = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('assets::type.name')) 
            ->view('assets::type.create', true) 
            ->data(compact('type'))
            ->output();
    }

    /**
     * Create new type.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TypeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $type                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('assets::type.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('assets/type/' . $type->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/assets/type'))
                ->redirect();
        }

    }

    /**
     * Show type for editing.
     *
     * @param Request $request
     * @param Model   $type
     *
     * @return Response
     */
    public function edit(TypeRequest $request, Type $type)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('assets::type.name'))
            ->view('assets::type.edit', true)
            ->data(compact('type'))
            ->output();
    }

    /**
     * Update the type.
     *
     * @param Request $request
     * @param Model   $type
     *
     * @return Response
     */
    public function update(TypeRequest $request, Type $type)
    {
        try {
            $attributes = $request->all();

            $type->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('assets::type.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('assets/type/' . $type->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('assets/type/' . $type->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the type.
     *
     * @param Model   $type
     *
     * @return Response
     */
    public function destroy(TypeRequest $request, Type $type)
    {
        try {

            $type->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('assets::type.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('assets/type/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('assets/type/' . $type->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple type.
     *
     * @param Model   $type
     *
     * @return Response
     */
    public function delete(TypeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('assets::type.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('assets/type'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/assets/type'))
                ->redirect();
        }

    }

    /**
     * Restore deleted types.
     *
     * @param Model   $type
     *
     * @return Response
     */
    public function restore(TypeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('assets::type.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/assets/type'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/assets/type/'))
                ->redirect();
        }

    }

}
